# cnorm.awk
#
# Thu Feb 21 09:42:55  2002
# 
# can be used to normalize columns to [0 1]
# 
# assuming formatted data (e.g. the two following lines from toinput.awk)
# min1 min2 min3 ... minn
# max1 max2 max3 ... maxn
# x1 x2 x3 ... xn c
# normalizing columns
#
# Example: 
#  awk 'NR==4 || NR==5' y.dat | awk -f cnorm.awk - x.dat > z.dat
# 
#   y.dat contains the error-channel output of toinput.awk
#   x.dat contains the standard-channel output of toinput.awk
#   z.dat becomes the normalized data


BEGIN {
  getline; for(i = 1; i <= NF; i++) min[i] = $i + 0.
  getline; for(i = 1; i <= NF; i++) max[i] = $i - min[i]
}

{
  if($0 ~ /(^#)|(^[ \t]*$)/) 
    printf("\n")
  else 
  {
    for(i = 1; i < NF; i++) 
      printf("%f ", (max[i] <= 0.) ? $i : ($i - min[i]) / max[i])
    print $(NF)
  }
}
