/*
 #   Supervised Relevance Neural Gas (srng) package.
 #   Copyright (C) 2002  Marc Strickert. (mstricke@uos.de)
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, version 2 of the License.
 #
 #   This program is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #   GNU General Public License (LICENSE.txt) for more details.
 #
 #   Marc Strickert, 25 February 2002
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <float.h>

#include <time.h>


/* minimum neighborhood size */
#define NEIGEPS 1e-5

/* Ignore-less-than-speedup - threshold*/
#define NEIGSPEEDTHRESH 0.025

/* denominator threshold */
#define EPS 1e-13

/* What it says :-) */
#define MAX_CLASSES 2048



/************** the following is of minor interest ****************************/

#define error(X) fprintf(stderr, (X)), exit(1)
#define panic(X) ptfree(), pattern_free(pattern_length), error((X))
#define frand() ((double) rand() / ((double)RAND_MAX + 1.0))
#define irand(X) ((int)((double)(X) * frand()))
      

typedef struct PROTOTYPE_TYPE *PROTOTYPE;

typedef struct PROTOTYPE_TYPE {
  int      cls;                     /* class */
  double   dist;                    /* distance to pattern */
  double   *coord;                  /* codebook-coordinates */
  double   *coord_shaddow;          /* double buffer prototype-coordinates */
  double   *weights;                /* p.-t. local lambdas ? */
} PROTOTYPE_TYPE;

PROTOTYPE *prototypes;

int prototypes_no, 
    prototypes_maxperclass,
    *prototypes_no_class,
    prototypes_lastindex;


double *global_lambda,   /* still global... */
       *block_weight;    /* weighting factors for unary coding */

double ng_fader;         /* typically:0.25 sgd slope-max pos or <=0.: no fade */
int blocks_no,           /* number of blocks */
    *block_len;          /* for unary coding */

double **pattern;        /* contains float data from STDIN */

int *shuffle_index;      /* helper for data shuffling */

       
int pattern_length,      /* Length of data series */
    pattern_dimension;   /* ...     */


int training_cycles,     /* training_cycles so far */
    total_iterations;    /* total number of cycles */

double rate_eta,                       /* coordinate adaptation rate */
       rate_alpha,                     /* lambda adaptation rate */
       rate_alphadecay,                /* lambda weight decay */
       rate_neighborhood_annealing;    /* about 0.9x decay rate */

int moduloprint;         /* if > 0 : print learning trace */

unsigned randseed;
       
double neighborhood_size;


/******************************************************************************/

unsigned randomize();

void pattern_free(int);

/******************************************************************************/



unsigned randomize() {

  int srd, tmp;  /* auto-variable: not initialized -> random */
                 /* Compiler Warnings are welcome here ! */
  time_t t;
  struct tm *ts;
  
  time(&t);              
  ts = localtime(&t);    

  tmp = ts->tm_mday | (ts->tm_min << 6) | (ts->tm_sec << 12);

  if(srd == 0) 
    srd |= tmp;
  else
    srd *= tmp;
  
  srand((unsigned)srd); 

  return (unsigned)srd;
}
/******************************************************************************/
