# toinput.awk
#
# Thu Feb 21 09:19:01  2002
#
# Format data to x1 x2 x3 ... xn c
#
# typical use: 
#   awk -vSWAP=n -Fs -f toinput.awk source.dat > dest.dat
#
# SWAP: column n corresponding to the class label, to be swapped to last column
#       default: SWAP=0 means class label already in last column
# s: field separator between columns: default: ' '. Other ';', ',', '|', or ''
#
# print to stderr:
# number_lines number_columns
# min1 min2 min3 ... minn
# max1 max2 max3 ... maxn
# 1 -> c1 class 
# 2 -> c2 transscript ...


BEGIN {
  TOSTDERR = "cat >&2"
  clscnt = -1
}


!/(^#)|(^[ \t]*$)/ {

  if(oldnf != "" && NF != oldnf) {
    print "Dimension mismatch in line " NR " !" | TOSTDERR
    exit(1)
  }

  if(SWAP > 0 && SWAP < NF) {
    u = $(SWAP)
    for(i = SWAP+1; i <= NF; i++) $(i-1) = $i
    $(NF) = u
  }
  
  ind = $(NF)
  
  if(!(ind in classes)) {
     classes[ind] = ind
     ++clscnt
     clsnam[clscnt] = ind
  }

  $(NF) = clscnt

  for(i=1; i < NF; i++) { 
    if(!(i in max)) max[i] = -(min[i] = 1e10)
    if(0.+$i > max[i]) max[i] = $i + 0.
    if(0.+$i < min[i]) min[i] = $i + 0.
  }

  print

  oldnf = NF
  ++line
}


END {

  print "# set size (patterns, dimensions [without class])" | TOSTDERR
  print line, oldnf-1 | TOSTDERR
  
  mi = ma = ""
  
  for(i = 1; i < oldnf; i++) {
    mi = mi ((i == 1) ? "" : " ") min[i]
    ma = ma ((i == 1) ? "" : " ") max[i]
  }
  
  print "# minima (1st line) and maxima (2nd line) in columns" | TOSTDERR
  print mi | TOSTDERR
  print ma | TOSTDERR

  print "# classname to class number transscription" | TOSTDERR
  for(i = 0; i <= clscnt; i++) 
    print clsnam[i]  "=" i | TOSTDERR
}
